#version 120

#define YELLOW					// Adds a slight yellow tint to everything
#define TONEMAP					// Modified ACES Tonemap
#define NIGHT_EYE				// Adds a blue tint to dark areas

#define BLOOM					// Makes everything more smooth and bright areas more glowy
#define BLOOM_QUALITY 10		// High performance impact! [4 5 6 7 8 9 10 11 12 13 14 15 16] 
#define BLOOM_RANGE 8			// No performance impact [1 2 4 8 12 16 20 24 32 40 48 64]
#define BLOOM_STRENGTH 0.16		// No performance impact [0.02 0.04 0.06 0.08 0.10 0.12 0.14 0.16 0.18 0.20 0.22 0.24 0.26 0.28 0.30 0.32]

#define GAMMA 0.50				// Higher values will make everything brighter [0.00 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.55 0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95 1.00]

uniform sampler2D colortex0;
uniform sampler2D depthtex0;

uniform float viewWidth;
uniform float viewHeight;

varying vec2 texcoord;

///////////////////////////////

float avg3(vec3 vec) {
	return (vec.x + vec.y + vec.z) / 3.0f;
}

float max3(vec3 vec) {
	return max(max(vec.x, vec.y), vec.z);
}

///////////////////////////////

void ACES(inout vec3 x) {
	// Tonemap by https://github.com/dmnsgn/glsl-tone-map
	/*
	Default values
	const float a = 2.51;
	const float b = 0.03;
	const float c = 2.43;
	const float d = 0.59;
	const float e = 0.14;
	
	Unchanged Colors
	const float a = 0.0;
	const float b = 1.0;
	const float c = 0.0;
	const float d = 0.0;
	const float e = 1.0;
	*/
	
	#ifdef TONEMAP
	float a,b,c,d,e,f;
	
	a = 1.4;
	b = 0.09;
	c = 1.0;
	d = 0.5;
	e = 0.15;
	
	x = clamp((x * (a * x + b)) / (x * (c * x + d) + e), 0.0, 1.0);
	#endif
}

///////////////////////////////

void Bloom(inout vec3 color) {
	#ifdef BLOOM
	vec3 blur = vec3(0.0f);
	
	int quality = BLOOM_QUALITY;
	float range = BLOOM_RANGE;
	
	float qh = quality / 2.0f - 0.5f;
	float allStrengths = 0.0f;
	for(int i=0; i < quality; i++) {
		for(int j=0; j < quality; j++) {
			vec2 offset = vec2((i - qh) * range / viewWidth, (j - qh) * range / viewHeight);
			
			float dist = 1.33f - distance(vec2(qh), vec2(i, j)) / qh;
			
			float strength = dist;
			
			blur += texture2D(colortex0, texcoord.xy + offset).rgb * strength;
			allStrengths += strength;
		}
	}
	
	blur /= allStrengths;
	float avg = avg3(blur);
	blur = mix(vec3(avg), blur, 2.0f);
	blur = clamp(blur, 0.0f, 1.0f);
	color = color.rgb * (1.0f - BLOOM_STRENGTH) + blur * BLOOM_STRENGTH;
	#endif
}

void NightEye(inout vec3 color) {
	#ifdef NIGHT_EYE
	float brightness = max3(color);
	float strength = clamp((pow(1.0f - brightness, 5)), 0.0f, 1.0f);
	vec3 cold = color * vec3(0.0f, 0.4f, 1.0f) * 1.5f;
	color = mix(color, cold, strength);
	#endif
}

void ColorConversion(inout vec3 color) {
	vec3 newColor;
	newColor.r = (1.086 * color.r) + (-0.072 * color.g) + (-0.014 * color.b);
	newColor.g = (0.097 * color.r) + (0.845 * color.g) + (0.058 * color.b);
	newColor.b = (-0.014 * color.r) + (-0.028 * color.g) + (1.042 * color.b);
	color = clamp(newColor, 0.0, 1.0);
}

void AdjustGamma(inout vec3 color) {
	color = pow(color, vec3(1.0 - GAMMA + 0.5));
}

void main() {
	vec3 color = texture2D(colortex0, texcoord.xy).rgb;
	
	Bloom(color);
	
	#ifdef YELLOW
	float strength = 0.1f;
	color = color * (vec3(1.0, 0.9f, 0.0f) * strength + vec3(1.0f)) * (1.0f - strength);
	#endif
	
	NightEye(color);
	ACES(color);
	AdjustGamma(color);
	ColorConversion(color);
	
	gl_FragColor = vec4(color, 1.0);
}